# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import diagnostics_pb2 as diagnostics__pb2
from . import empty_pb2 as empty__pb2


class DiagnosticsStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.ListPeers = channel.unary_unary(
        '/io.casperlabs.node.api.diagnostics.Diagnostics/ListPeers',
        request_serializer=empty__pb2.Empty.SerializeToString,
        response_deserializer=diagnostics__pb2.Peers.FromString,
        )
    self.ListDiscoveredPeers = channel.unary_unary(
        '/io.casperlabs.node.api.diagnostics.Diagnostics/ListDiscoveredPeers',
        request_serializer=empty__pb2.Empty.SerializeToString,
        response_deserializer=diagnostics__pb2.Peers.FromString,
        )
    self.GetProcessCpu = channel.unary_unary(
        '/io.casperlabs.node.api.diagnostics.Diagnostics/GetProcessCpu',
        request_serializer=empty__pb2.Empty.SerializeToString,
        response_deserializer=diagnostics__pb2.ProcessCpu.FromString,
        )
    self.GetMemoryUsage = channel.unary_unary(
        '/io.casperlabs.node.api.diagnostics.Diagnostics/GetMemoryUsage',
        request_serializer=empty__pb2.Empty.SerializeToString,
        response_deserializer=diagnostics__pb2.MemoryUsage.FromString,
        )
    self.GetGarbageCollectors = channel.unary_unary(
        '/io.casperlabs.node.api.diagnostics.Diagnostics/GetGarbageCollectors',
        request_serializer=empty__pb2.Empty.SerializeToString,
        response_deserializer=diagnostics__pb2.GarbageCollectors.FromString,
        )
    self.GetMemoryPools = channel.unary_unary(
        '/io.casperlabs.node.api.diagnostics.Diagnostics/GetMemoryPools',
        request_serializer=empty__pb2.Empty.SerializeToString,
        response_deserializer=diagnostics__pb2.MemoryPools.FromString,
        )
    self.GetThreads = channel.unary_unary(
        '/io.casperlabs.node.api.diagnostics.Diagnostics/GetThreads',
        request_serializer=empty__pb2.Empty.SerializeToString,
        response_deserializer=diagnostics__pb2.Threads.FromString,
        )
    self.GetNodeCoreMetrics = channel.unary_unary(
        '/io.casperlabs.node.api.diagnostics.Diagnostics/GetNodeCoreMetrics',
        request_serializer=empty__pb2.Empty.SerializeToString,
        response_deserializer=diagnostics__pb2.NodeCoreMetrics.FromString,
        )


class DiagnosticsServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def ListPeers(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ListDiscoveredPeers(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetProcessCpu(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetMemoryUsage(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetGarbageCollectors(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetMemoryPools(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetThreads(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetNodeCoreMetrics(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_DiagnosticsServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'ListPeers': grpc.unary_unary_rpc_method_handler(
          servicer.ListPeers,
          request_deserializer=empty__pb2.Empty.FromString,
          response_serializer=diagnostics__pb2.Peers.SerializeToString,
      ),
      'ListDiscoveredPeers': grpc.unary_unary_rpc_method_handler(
          servicer.ListDiscoveredPeers,
          request_deserializer=empty__pb2.Empty.FromString,
          response_serializer=diagnostics__pb2.Peers.SerializeToString,
      ),
      'GetProcessCpu': grpc.unary_unary_rpc_method_handler(
          servicer.GetProcessCpu,
          request_deserializer=empty__pb2.Empty.FromString,
          response_serializer=diagnostics__pb2.ProcessCpu.SerializeToString,
      ),
      'GetMemoryUsage': grpc.unary_unary_rpc_method_handler(
          servicer.GetMemoryUsage,
          request_deserializer=empty__pb2.Empty.FromString,
          response_serializer=diagnostics__pb2.MemoryUsage.SerializeToString,
      ),
      'GetGarbageCollectors': grpc.unary_unary_rpc_method_handler(
          servicer.GetGarbageCollectors,
          request_deserializer=empty__pb2.Empty.FromString,
          response_serializer=diagnostics__pb2.GarbageCollectors.SerializeToString,
      ),
      'GetMemoryPools': grpc.unary_unary_rpc_method_handler(
          servicer.GetMemoryPools,
          request_deserializer=empty__pb2.Empty.FromString,
          response_serializer=diagnostics__pb2.MemoryPools.SerializeToString,
      ),
      'GetThreads': grpc.unary_unary_rpc_method_handler(
          servicer.GetThreads,
          request_deserializer=empty__pb2.Empty.FromString,
          response_serializer=diagnostics__pb2.Threads.SerializeToString,
      ),
      'GetNodeCoreMetrics': grpc.unary_unary_rpc_method_handler(
          servicer.GetNodeCoreMetrics,
          request_deserializer=empty__pb2.Empty.FromString,
          response_serializer=diagnostics__pb2.NodeCoreMetrics.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'io.casperlabs.node.api.diagnostics.Diagnostics', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import consensus_pb2 as consensus__pb2
from . import empty_pb2 as empty__pb2
from . import gossiping_pb2 as gossiping__pb2


class GossipServiceStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.NewBlocks = channel.unary_unary(
        '/io.casperlabs.comm.gossiping.GossipService/NewBlocks',
        request_serializer=gossiping__pb2.NewBlocksRequest.SerializeToString,
        response_deserializer=gossiping__pb2.NewBlocksResponse.FromString,
        )
    self.StreamAncestorBlockSummaries = channel.unary_stream(
        '/io.casperlabs.comm.gossiping.GossipService/StreamAncestorBlockSummaries',
        request_serializer=gossiping__pb2.StreamAncestorBlockSummariesRequest.SerializeToString,
        response_deserializer=consensus__pb2.BlockSummary.FromString,
        )
    self.StreamDagTipBlockSummaries = channel.unary_stream(
        '/io.casperlabs.comm.gossiping.GossipService/StreamDagTipBlockSummaries',
        request_serializer=gossiping__pb2.StreamDagTipBlockSummariesRequest.SerializeToString,
        response_deserializer=consensus__pb2.BlockSummary.FromString,
        )
    self.StreamBlockSummaries = channel.unary_stream(
        '/io.casperlabs.comm.gossiping.GossipService/StreamBlockSummaries',
        request_serializer=gossiping__pb2.StreamBlockSummariesRequest.SerializeToString,
        response_deserializer=consensus__pb2.BlockSummary.FromString,
        )
    self.GetBlockChunked = channel.unary_stream(
        '/io.casperlabs.comm.gossiping.GossipService/GetBlockChunked',
        request_serializer=gossiping__pb2.GetBlockChunkedRequest.SerializeToString,
        response_deserializer=gossiping__pb2.Chunk.FromString,
        )
    self.GetGenesisCandidate = channel.unary_unary(
        '/io.casperlabs.comm.gossiping.GossipService/GetGenesisCandidate',
        request_serializer=gossiping__pb2.GetGenesisCandidateRequest.SerializeToString,
        response_deserializer=consensus__pb2.GenesisCandidate.FromString,
        )
    self.AddApproval = channel.unary_unary(
        '/io.casperlabs.comm.gossiping.GossipService/AddApproval',
        request_serializer=gossiping__pb2.AddApprovalRequest.SerializeToString,
        response_deserializer=empty__pb2.Empty.FromString,
        )


class GossipServiceServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def NewBlocks(self, request, context):
    """Notify the callee about new blocks being available on the caller.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def StreamAncestorBlockSummaries(self, request, context):
    """Retrieve the ancestors of certain blocks in the DAG; to be called repeatedly
    as necessary to synchronize DAGs between peers.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def StreamDagTipBlockSummaries(self, request, context):
    """Retrieve the tips of the DAG as the callee knows them, to kick off synchronization between peers.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def StreamBlockSummaries(self, request, context):
    """Retrieve arbitrary block summaries, if the callee knows about them.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetBlockChunked(self, request, context):
    """Retrieve an arbitrarily sized block as a stream of chunks, with optionally compressed content.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetGenesisCandidate(self, request, context):
    """Retrieve the Genesis candidate supported by this node. Every node should either produce one from specification,
    get one from its bootstrap, or have it persisted from earlier runs.
    While the node is initializing and it hasn't obtained the candidate yet it will return UNAVAILABLE.
    Once the node is serving a candidate identity, it should also be prepared to serve the full block on request.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def AddApproval(self, request, context):
    """Add a signature to the list of approvals of a given candidate. If the block hash in the request doesn't match
    the callee's candidate it will return INVALID_ARGUMENT, otherwise add the signature and forward it to its peers.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_GossipServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'NewBlocks': grpc.unary_unary_rpc_method_handler(
          servicer.NewBlocks,
          request_deserializer=gossiping__pb2.NewBlocksRequest.FromString,
          response_serializer=gossiping__pb2.NewBlocksResponse.SerializeToString,
      ),
      'StreamAncestorBlockSummaries': grpc.unary_stream_rpc_method_handler(
          servicer.StreamAncestorBlockSummaries,
          request_deserializer=gossiping__pb2.StreamAncestorBlockSummariesRequest.FromString,
          response_serializer=consensus__pb2.BlockSummary.SerializeToString,
      ),
      'StreamDagTipBlockSummaries': grpc.unary_stream_rpc_method_handler(
          servicer.StreamDagTipBlockSummaries,
          request_deserializer=gossiping__pb2.StreamDagTipBlockSummariesRequest.FromString,
          response_serializer=consensus__pb2.BlockSummary.SerializeToString,
      ),
      'StreamBlockSummaries': grpc.unary_stream_rpc_method_handler(
          servicer.StreamBlockSummaries,
          request_deserializer=gossiping__pb2.StreamBlockSummariesRequest.FromString,
          response_serializer=consensus__pb2.BlockSummary.SerializeToString,
      ),
      'GetBlockChunked': grpc.unary_stream_rpc_method_handler(
          servicer.GetBlockChunked,
          request_deserializer=gossiping__pb2.GetBlockChunkedRequest.FromString,
          response_serializer=gossiping__pb2.Chunk.SerializeToString,
      ),
      'GetGenesisCandidate': grpc.unary_unary_rpc_method_handler(
          servicer.GetGenesisCandidate,
          request_deserializer=gossiping__pb2.GetGenesisCandidateRequest.FromString,
          response_serializer=consensus__pb2.GenesisCandidate.SerializeToString,
      ),
      'AddApproval': grpc.unary_unary_rpc_method_handler(
          servicer.AddApproval,
          request_deserializer=gossiping__pb2.AddApprovalRequest.FromString,
          response_serializer=empty__pb2.Empty.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'io.casperlabs.comm.gossiping.GossipService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: info.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import consensus_pb2 as consensus__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='info.proto',
  package='io.casperlabs.casper.consensus.info',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\ninfo.proto\x12#io.casperlabs.casper.consensus.info\x1a\x0f\x63onsensus.proto\"\xf8\x02\n\tBlockInfo\x12=\n\x07summary\x18\x01 \x01(\x0b\x32,.io.casperlabs.casper.consensus.BlockSummary\x12\x45\n\x06status\x18\x02 \x01(\x0b\x32\x35.io.casperlabs.casper.consensus.info.BlockInfo.Status\x1a\xc7\x01\n\x06Status\x12\x17\n\x0f\x66\x61ult_tolerance\x18\x01 \x01(\x02\x12J\n\x05stats\x18\x02 \x01(\x0b\x32;.io.casperlabs.casper.consensus.info.BlockInfo.Status.Stats\x1aX\n\x05Stats\x12\x18\n\x10\x62lock_size_bytes\x18\x01 \x01(\r\x12\x1a\n\x12\x64\x65ploy_error_count\x18\x02 \x01(\r\x12\x19\n\x11\x64\x65ploy_cost_total\x18\x03 \x01(\x04\"\x1b\n\x04View\x12\t\n\x05\x42\x41SIC\x10\x00\x12\x08\n\x04\x46ULL\x10\x01\"\xca\x04\n\nDeployInfo\x12\x36\n\x06\x64\x65ploy\x18\x01 \x01(\x0b\x32&.io.casperlabs.casper.consensus.Deploy\x12\\\n\x12processing_results\x18\x02 \x03(\x0b\x32@.io.casperlabs.casper.consensus.info.DeployInfo.ProcessingResult\x12\x46\n\x06status\x18\x03 \x01(\x0b\x32\x36.io.casperlabs.casper.consensus.info.DeployInfo.Status\x1a\x8d\x01\n\x10ProcessingResult\x12\x42\n\nblock_info\x18\x01 \x01(\x0b\x32..io.casperlabs.casper.consensus.info.BlockInfo\x12\x0c\n\x04\x63ost\x18\x02 \x01(\x04\x12\x10\n\x08is_error\x18\x03 \x01(\x08\x12\x15\n\rerror_message\x18\x04 \x01(\t\x1a_\n\x06Status\x12\x44\n\x05state\x18\x01 \x01(\x0e\x32\x35.io.casperlabs.casper.consensus.info.DeployInfo.State\x12\x0f\n\x07message\x18\x02 \x01(\t\"\x1b\n\x04View\x12\t\n\x05\x42\x41SIC\x10\x00\x12\x08\n\x04\x46ULL\x10\x01\"P\n\x05State\x12\r\n\tUNDEFINED\x10\x00\x12\x0b\n\x07PENDING\x10\x01\x12\r\n\tPROCESSED\x10\x02\x12\r\n\tFINALIZED\x10\x03\x12\r\n\tDISCARDED\x10\x04\x62\x06proto3')
  ,
  dependencies=[consensus__pb2.DESCRIPTOR,])



_BLOCKINFO_VIEW = _descriptor.EnumDescriptor(
  name='View',
  full_name='io.casperlabs.casper.consensus.info.BlockInfo.View',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='BASIC', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FULL', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=418,
  serialized_end=445,
)
_sym_db.RegisterEnumDescriptor(_BLOCKINFO_VIEW)

_DEPLOYINFO_VIEW = _descriptor.EnumDescriptor(
  name='View',
  full_name='io.casperlabs.casper.consensus.info.DeployInfo.View',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='BASIC', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FULL', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=418,
  serialized_end=445,
)
_sym_db.RegisterEnumDescriptor(_DEPLOYINFO_VIEW)

_DEPLOYINFO_STATE = _descriptor.EnumDescriptor(
  name='State',
  full_name='io.casperlabs.casper.consensus.info.DeployInfo.State',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNDEFINED', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PENDING', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROCESSED', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FINALIZED', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DISCARDED', index=4, number=4,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=954,
  serialized_end=1034,
)
_sym_db.RegisterEnumDescriptor(_DEPLOYINFO_STATE)


_BLOCKINFO_STATUS_STATS = _descriptor.Descriptor(
  name='Stats',
  full_name='io.casperlabs.casper.consensus.info.BlockInfo.Status.Stats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_size_bytes', full_name='io.casperlabs.casper.consensus.info.BlockInfo.Status.Stats.block_size_bytes', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deploy_error_count', full_name='io.casperlabs.casper.consensus.info.BlockInfo.Status.Stats.deploy_error_count', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deploy_cost_total', full_name='io.casperlabs.casper.consensus.info.BlockInfo.Status.Stats.deploy_cost_total', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=328,
  serialized_end=416,
)

_BLOCKINFO_STATUS = _descriptor.Descriptor(
  name='Status',
  full_name='io.casperlabs.casper.consensus.info.BlockInfo.Status',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='fault_tolerance', full_name='io.casperlabs.casper.consensus.info.BlockInfo.Status.fault_tolerance', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stats', full_name='io.casperlabs.casper.consensus.info.BlockInfo.Status.stats', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_BLOCKINFO_STATUS_STATS, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=217,
  serialized_end=416,
)

_BLOCKINFO = _descriptor.Descriptor(
  name='BlockInfo',
  full_name='io.casperlabs.casper.consensus.info.BlockInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='summary', full_name='io.casperlabs.casper.consensus.info.BlockInfo.summary', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='io.casperlabs.casper.consensus.info.BlockInfo.status', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_BLOCKINFO_STATUS, ],
  enum_types=[
    _BLOCKINFO_VIEW,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=69,
  serialized_end=445,
)


_DEPLOYINFO_PROCESSINGRESULT = _descriptor.Descriptor(
  name='ProcessingResult',
  full_name='io.casperlabs.casper.consensus.info.DeployInfo.ProcessingResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_info', full_name='io.casperlabs.casper.consensus.info.DeployInfo.ProcessingResult.block_info', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cost', full_name='io.casperlabs.casper.consensus.info.DeployInfo.ProcessingResult.cost', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='is_error', full_name='io.casperlabs.casper.consensus.info.DeployInfo.ProcessingResult.is_error', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error_message', full_name='io.casperlabs.casper.consensus.info.DeployInfo.ProcessingResult.error_message', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=685,
  serialized_end=826,
)

_DEPLOYINFO_STATUS = _descriptor.Descriptor(
  name='Status',
  full_name='io.casperlabs.casper.consensus.info.DeployInfo.Status',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='io.casperlabs.casper.consensus.info.DeployInfo.Status.state', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message', full_name='io.casperlabs.casper.consensus.info.DeployInfo.Status.message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=828,
  serialized_end=923,
)

_DEPLOYINFO = _descriptor.Descriptor(
  name='DeployInfo',
  full_name='io.casperlabs.casper.consensus.info.DeployInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='deploy', full_name='io.casperlabs.casper.consensus.info.DeployInfo.deploy', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='processing_results', full_name='io.casperlabs.casper.consensus.info.DeployInfo.processing_results', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='io.casperlabs.casper.consensus.info.DeployInfo.status', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_DEPLOYINFO_PROCESSINGRESULT, _DEPLOYINFO_STATUS, ],
  enum_types=[
    _DEPLOYINFO_VIEW,
    _DEPLOYINFO_STATE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=448,
  serialized_end=1034,
)

_BLOCKINFO_STATUS_STATS.containing_type = _BLOCKINFO_STATUS
_BLOCKINFO_STATUS.fields_by_name['stats'].message_type = _BLOCKINFO_STATUS_STATS
_BLOCKINFO_STATUS.containing_type = _BLOCKINFO
_BLOCKINFO.fields_by_name['summary'].message_type = consensus__pb2._BLOCKSUMMARY
_BLOCKINFO.fields_by_name['status'].message_type = _BLOCKINFO_STATUS
_BLOCKINFO_VIEW.containing_type = _BLOCKINFO
_DEPLOYINFO_PROCESSINGRESULT.fields_by_name['block_info'].message_type = _BLOCKINFO
_DEPLOYINFO_PROCESSINGRESULT.containing_type = _DEPLOYINFO
_DEPLOYINFO_STATUS.fields_by_name['state'].enum_type = _DEPLOYINFO_STATE
_DEPLOYINFO_STATUS.containing_type = _DEPLOYINFO
_DEPLOYINFO.fields_by_name['deploy'].message_type = consensus__pb2._DEPLOY
_DEPLOYINFO.fields_by_name['processing_results'].message_type = _DEPLOYINFO_PROCESSINGRESULT
_DEPLOYINFO.fields_by_name['status'].message_type = _DEPLOYINFO_STATUS
_DEPLOYINFO_VIEW.containing_type = _DEPLOYINFO
_DEPLOYINFO_STATE.containing_type = _DEPLOYINFO
DESCRIPTOR.message_types_by_name['BlockInfo'] = _BLOCKINFO
DESCRIPTOR.message_types_by_name['DeployInfo'] = _DEPLOYINFO
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

BlockInfo = _reflection.GeneratedProtocolMessageType('BlockInfo', (_message.Message,), {

  'Status' : _reflection.GeneratedProtocolMessageType('Status', (_message.Message,), {

    'Stats' : _reflection.GeneratedProtocolMessageType('Stats', (_message.Message,), {
      'DESCRIPTOR' : _BLOCKINFO_STATUS_STATS,
      '__module__' : 'info_pb2'
      # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.BlockInfo.Status.Stats)
      })
    ,
    'DESCRIPTOR' : _BLOCKINFO_STATUS,
    '__module__' : 'info_pb2'
    # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.BlockInfo.Status)
    })
  ,
  'DESCRIPTOR' : _BLOCKINFO,
  '__module__' : 'info_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.BlockInfo)
  })
_sym_db.RegisterMessage(BlockInfo)
_sym_db.RegisterMessage(BlockInfo.Status)
_sym_db.RegisterMessage(BlockInfo.Status.Stats)

DeployInfo = _reflection.GeneratedProtocolMessageType('DeployInfo', (_message.Message,), {

  'ProcessingResult' : _reflection.GeneratedProtocolMessageType('ProcessingResult', (_message.Message,), {
    'DESCRIPTOR' : _DEPLOYINFO_PROCESSINGRESULT,
    '__module__' : 'info_pb2'
    # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.DeployInfo.ProcessingResult)
    })
  ,

  'Status' : _reflection.GeneratedProtocolMessageType('Status', (_message.Message,), {
    'DESCRIPTOR' : _DEPLOYINFO_STATUS,
    '__module__' : 'info_pb2'
    # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.DeployInfo.Status)
    })
  ,
  'DESCRIPTOR' : _DEPLOYINFO,
  '__module__' : 'info_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.casper.consensus.info.DeployInfo)
  })
_sym_db.RegisterMessage(DeployInfo)
_sym_db.RegisterMessage(DeployInfo.ProcessingResult)
_sym_db.RegisterMessage(DeployInfo.Status)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: diagnostics.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import empty_pb2 as empty__pb2
from . import wrappers_pb2 as wrappers__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='diagnostics.proto',
  package='io.casperlabs.node.api.diagnostics',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x11\x64iagnostics.proto\x12\"io.casperlabs.node.api.diagnostics\x1a\x0b\x65mpty.proto\x1a\x0ewrappers.proto\"c\n\nProcessCpu\x12*\n\x04load\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12)\n\x04time\x18\x02 \x01(\x0b\x32\x1b.google.protobuf.Int64Value\"\x84\x01\n\x0bMemoryUsage\x12\x38\n\x04heap\x18\x01 \x01(\x0b\x32*.io.casperlabs.node.api.diagnostics.Memory\x12;\n\x07nonHeap\x18\x02 \x01(\x0b\x32*.io.casperlabs.node.api.diagnostics.Memory\"a\n\x06Memory\x12\x11\n\tcommitted\x18\x01 \x01(\x03\x12\x0c\n\x04init\x18\x02 \x01(\x03\x12(\n\x03max\x18\x03 \x01(\x0b\x32\x1b.google.protobuf.Int64Value\x12\x0c\n\x04used\x18\x04 \x01(\x03\"\xe4\x01\n\x10GarbageCollector\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x18\n\x10totalCollections\x18\x02 \x01(\x03\x12\x1b\n\x13totalCollectionTime\x18\x03 \x01(\x03\x12.\n\tstartTime\x18\x04 \x01(\x0b\x32\x1b.google.protobuf.Int64Value\x12,\n\x07\x65ndTime\x18\x05 \x01(\x0b\x32\x1b.google.protobuf.Int64Value\x12-\n\x08\x64uration\x18\x06 \x01(\x0b\x32\x1b.google.protobuf.Int64Value\"\xa6\x01\n\nMemoryPool\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08poolType\x18\x02 \x01(\t\x12\x39\n\x05usage\x18\x03 \x01(\x0b\x32*.io.casperlabs.node.api.diagnostics.Memory\x12=\n\tpeakUsage\x18\x04 \x01(\x0b\x32*.io.casperlabs.node.api.diagnostics.Memory\"s\n\x07Threads\x12\x13\n\x0bthreadCount\x18\x01 \x01(\x05\x12\x19\n\x11\x64\x61\x65monThreadCount\x18\x02 \x01(\x05\x12\x17\n\x0fpeakThreadCount\x18\x03 \x01(\x05\x12\x1f\n\x17totalStartedThreadCount\x18\x04 \x01(\x03\"\xfd\x01\n\x0fNodeCoreMetrics\x12\x19\n\x11pingReceiverCount\x18\x01 \x01(\x03\x12\x1b\n\x13lookupReceiverCount\x18\x02 \x01(\x03\x12\x1f\n\x17\x64isconnectReceiverCount\x18\x03 \x01(\x03\x12\x10\n\x08\x63onnects\x18\x04 \x01(\x03\x12+\n#p2pEncryptionHandshakeReceiverCount\x18\x05 \x01(\x03\x12)\n!p2pProtocolHandshakeReceiverCount\x18\x06 \x01(\x03\x12\r\n\x05peers\x18\x07 \x01(\x03\x12\x0c\n\x04\x66rom\x18\x08 \x01(\x03\x12\n\n\x02to\x18\t \x01(\x03\"d\n\x11GarbageCollectors\x12O\n\x11garbageCollectors\x18\x01 \x03(\x0b\x32\x34.io.casperlabs.node.api.diagnostics.GarbageCollector\"R\n\x0bMemoryPools\x12\x43\n\x0bmemoryPools\x18\x01 \x03(\x0b\x32..io.casperlabs.node.api.diagnostics.MemoryPool\"@\n\x05Peers\x12\x37\n\x05peers\x18\x01 \x03(\x0b\x32(.io.casperlabs.node.api.diagnostics.Peer\"/\n\x04Peer\x12\x0c\n\x04host\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\x05\x12\x0b\n\x03key\x18\x03 \x01(\x0c\x32\xe3\x05\n\x0b\x44iagnostics\x12N\n\tListPeers\x12\x16.google.protobuf.Empty\x1a).io.casperlabs.node.api.diagnostics.Peers\x12X\n\x13ListDiscoveredPeers\x12\x16.google.protobuf.Empty\x1a).io.casperlabs.node.api.diagnostics.Peers\x12W\n\rGetProcessCpu\x12\x16.google.protobuf.Empty\x1a..io.casperlabs.node.api.diagnostics.ProcessCpu\x12Y\n\x0eGetMemoryUsage\x12\x16.google.protobuf.Empty\x1a/.io.casperlabs.node.api.diagnostics.MemoryUsage\x12\x65\n\x14GetGarbageCollectors\x12\x16.google.protobuf.Empty\x1a\x35.io.casperlabs.node.api.diagnostics.GarbageCollectors\x12Y\n\x0eGetMemoryPools\x12\x16.google.protobuf.Empty\x1a/.io.casperlabs.node.api.diagnostics.MemoryPools\x12Q\n\nGetThreads\x12\x16.google.protobuf.Empty\x1a+.io.casperlabs.node.api.diagnostics.Threads\x12\x61\n\x12GetNodeCoreMetrics\x12\x16.google.protobuf.Empty\x1a\x33.io.casperlabs.node.api.diagnostics.NodeCoreMetricsb\x06proto3')
  ,
  dependencies=[empty__pb2.DESCRIPTOR,wrappers__pb2.DESCRIPTOR,])




_PROCESSCPU = _descriptor.Descriptor(
  name='ProcessCpu',
  full_name='io.casperlabs.node.api.diagnostics.ProcessCpu',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='load', full_name='io.casperlabs.node.api.diagnostics.ProcessCpu.load', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='time', full_name='io.casperlabs.node.api.diagnostics.ProcessCpu.time', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=86,
  serialized_end=185,
)


_MEMORYUSAGE = _descriptor.Descriptor(
  name='MemoryUsage',
  full_name='io.casperlabs.node.api.diagnostics.MemoryUsage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='heap', full_name='io.casperlabs.node.api.diagnostics.MemoryUsage.heap', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nonHeap', full_name='io.casperlabs.node.api.diagnostics.MemoryUsage.nonHeap', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=188,
  serialized_end=320,
)


_MEMORY = _descriptor.Descriptor(
  name='Memory',
  full_name='io.casperlabs.node.api.diagnostics.Memory',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='committed', full_name='io.casperlabs.node.api.diagnostics.Memory.committed', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='init', full_name='io.casperlabs.node.api.diagnostics.Memory.init', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max', full_name='io.casperlabs.node.api.diagnostics.Memory.max', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='used', full_name='io.casperlabs.node.api.diagnostics.Memory.used', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=322,
  serialized_end=419,
)


_GARBAGECOLLECTOR = _descriptor.Descriptor(
  name='GarbageCollector',
  full_name='io.casperlabs.node.api.diagnostics.GarbageCollector',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='io.casperlabs.node.api.diagnostics.GarbageCollector.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='totalCollections', full_name='io.casperlabs.node.api.diagnostics.GarbageCollector.totalCollections', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='totalCollectionTime', full_name='io.casperlabs.node.api.diagnostics.GarbageCollector.totalCollectionTime', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='startTime', full_name='io.casperlabs.node.api.diagnostics.GarbageCollector.startTime', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='endTime', full_name='io.casperlabs.node.api.diagnostics.GarbageCollector.endTime', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='duration', full_name='io.casperlabs.node.api.diagnostics.GarbageCollector.duration', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=422,
  serialized_end=650,
)


_MEMORYPOOL = _descriptor.Descriptor(
  name='MemoryPool',
  full_name='io.casperlabs.node.api.diagnostics.MemoryPool',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='io.casperlabs.node.api.diagnostics.MemoryPool.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='poolType', full_name='io.casperlabs.node.api.diagnostics.MemoryPool.poolType', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='usage', full_name='io.casperlabs.node.api.diagnostics.MemoryPool.usage', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='peakUsage', full_name='io.casperlabs.node.api.diagnostics.MemoryPool.peakUsage', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=653,
  serialized_end=819,
)


_THREADS = _descriptor.Descriptor(
  name='Threads',
  full_name='io.casperlabs.node.api.diagnostics.Threads',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='threadCount', full_name='io.casperlabs.node.api.diagnostics.Threads.threadCount', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='daemonThreadCount', full_name='io.casperlabs.node.api.diagnostics.Threads.daemonThreadCount', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='peakThreadCount', full_name='io.casperlabs.node.api.diagnostics.Threads.peakThreadCount', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='totalStartedThreadCount', full_name='io.casperlabs.node.api.diagnostics.Threads.totalStartedThreadCount', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=821,
  serialized_end=936,
)


_NODECOREMETRICS = _descriptor.Descriptor(
  name='NodeCoreMetrics',
  full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pingReceiverCount', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.pingReceiverCount', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lookupReceiverCount', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.lookupReceiverCount', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='disconnectReceiverCount', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.disconnectReceiverCount', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='connects', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.connects', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='p2pEncryptionHandshakeReceiverCount', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.p2pEncryptionHandshakeReceiverCount', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='p2pProtocolHandshakeReceiverCount', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.p2pProtocolHandshakeReceiverCount', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='peers', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.peers', index=6,
      number=7, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='from', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.from', index=7,
      number=8, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='to', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.to', index=8,
      number=9, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=939,
  serialized_end=1192,
)


_GARBAGECOLLECTORS = _descriptor.Descriptor(
  name='GarbageCollectors',
  full_name='io.casperlabs.node.api.diagnostics.GarbageCollectors',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='garbageCollectors', full_name='io.casperlabs.node.api.diagnostics.GarbageCollectors.garbageCollectors', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1194,
  serialized_end=1294,
)


_MEMORYPOOLS = _descriptor.Descriptor(
  name='MemoryPools',
  full_name='io.casperlabs.node.api.diagnostics.MemoryPools',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='memoryPools', full_name='io.casperlabs.node.api.diagnostics.MemoryPools.memoryPools', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1296,
  serialized_end=1378,
)


_PEERS = _descriptor.Descriptor(
  name='Peers',
  full_name='io.casperlabs.node.api.diagnostics.Peers',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='peers', full_name='io.casperlabs.node.api.diagnostics.Peers.peers', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1380,
  serialized_end=1444,
)


_PEER = _descriptor.Descriptor(
  name='Peer',
  full_name='io.casperlabs.node.api.diagnostics.Peer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='host', full_name='io.casperlabs.node.api.diagnostics.Peer.host', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='port', full_name='io.casperlabs.node.api.diagnostics.Peer.port', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='key', full_name='io.casperlabs.node.api.diagnostics.Peer.key', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1446,
  serialized_end=1493,
)

_PROCESSCPU.fields_by_name['load'].message_type = wrappers__pb2._DOUBLEVALUE
_PROCESSCPU.fields_by_name['time'].message_type = wrappers__pb2._INT64VALUE
_MEMORYUSAGE.fields_by_name['heap'].message_type = _MEMORY
_MEMORYUSAGE.fields_by_name['nonHeap'].message_type = _MEMORY
_MEMORY.fields_by_name['max'].message_type = wrappers__pb2._INT64VALUE
_GARBAGECOLLECTOR.fields_by_name['startTime'].message_type = wrappers__pb2._INT64VALUE
_GARBAGECOLLECTOR.fields_by_name['endTime'].message_type = wrappers__pb2._INT64VALUE
_GARBAGECOLLECTOR.fields_by_name['duration'].message_type = wrappers__pb2._INT64VALUE
_MEMORYPOOL.fields_by_name['usage'].message_type = _MEMORY
_MEMORYPOOL.fields_by_name['peakUsage'].message_type = _MEMORY
_GARBAGECOLLECTORS.fields_by_name['garbageCollectors'].message_type = _GARBAGECOLLECTOR
_MEMORYPOOLS.fields_by_name['memoryPools'].message_type = _MEMORYPOOL
_PEERS.fields_by_name['peers'].message_type = _PEER
DESCRIPTOR.message_types_by_name['ProcessCpu'] = _PROCESSCPU
DESCRIPTOR.message_types_by_name['MemoryUsage'] = _MEMORYUSAGE
DESCRIPTOR.message_types_by_name['Memory'] = _MEMORY
DESCRIPTOR.message_types_by_name['GarbageCollector'] = _GARBAGECOLLECTOR
DESCRIPTOR.message_types_by_name['MemoryPool'] = _MEMORYPOOL
DESCRIPTOR.message_types_by_name['Threads'] = _THREADS
DESCRIPTOR.message_types_by_name['NodeCoreMetrics'] = _NODECOREMETRICS
DESCRIPTOR.message_types_by_name['GarbageCollectors'] = _GARBAGECOLLECTORS
DESCRIPTOR.message_types_by_name['MemoryPools'] = _MEMORYPOOLS
DESCRIPTOR.message_types_by_name['Peers'] = _PEERS
DESCRIPTOR.message_types_by_name['Peer'] = _PEER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ProcessCpu = _reflection.GeneratedProtocolMessageType('ProcessCpu', (_message.Message,), {
  'DESCRIPTOR' : _PROCESSCPU,
  '__module__' : 'diagnostics_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.diagnostics.ProcessCpu)
  })
_sym_db.RegisterMessage(ProcessCpu)

MemoryUsage = _reflection.GeneratedProtocolMessageType('MemoryUsage', (_message.Message,), {
  'DESCRIPTOR' : _MEMORYUSAGE,
  '__module__' : 'diagnostics_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.diagnostics.MemoryUsage)
  })
_sym_db.RegisterMessage(MemoryUsage)

Memory = _reflection.GeneratedProtocolMessageType('Memory', (_message.Message,), {
  'DESCRIPTOR' : _MEMORY,
  '__module__' : 'diagnostics_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.diagnostics.Memory)
  })
_sym_db.RegisterMessage(Memory)

GarbageCollector = _reflection.GeneratedProtocolMessageType('GarbageCollector', (_message.Message,), {
  'DESCRIPTOR' : _GARBAGECOLLECTOR,
  '__module__' : 'diagnostics_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.diagnostics.GarbageCollector)
  })
_sym_db.RegisterMessage(GarbageCollector)

MemoryPool = _reflection.GeneratedProtocolMessageType('MemoryPool', (_message.Message,), {
  'DESCRIPTOR' : _MEMORYPOOL,
  '__module__' : 'diagnostics_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.diagnostics.MemoryPool)
  })
_sym_db.RegisterMessage(MemoryPool)

Threads = _reflection.GeneratedProtocolMessageType('Threads', (_message.Message,), {
  'DESCRIPTOR' : _THREADS,
  '__module__' : 'diagnostics_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.diagnostics.Threads)
  })
_sym_db.RegisterMessage(Threads)

NodeCoreMetrics = _reflection.GeneratedProtocolMessageType('NodeCoreMetrics', (_message.Message,), {
  'DESCRIPTOR' : _NODECOREMETRICS,
  '__module__' : 'diagnostics_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.diagnostics.NodeCoreMetrics)
  })
_sym_db.RegisterMessage(NodeCoreMetrics)

GarbageCollectors = _reflection.GeneratedProtocolMessageType('GarbageCollectors', (_message.Message,), {
  'DESCRIPTOR' : _GARBAGECOLLECTORS,
  '__module__' : 'diagnostics_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.diagnostics.GarbageCollectors)
  })
_sym_db.RegisterMessage(GarbageCollectors)

MemoryPools = _reflection.GeneratedProtocolMessageType('MemoryPools', (_message.Message,), {
  'DESCRIPTOR' : _MEMORYPOOLS,
  '__module__' : 'diagnostics_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.diagnostics.MemoryPools)
  })
_sym_db.RegisterMessage(MemoryPools)

Peers = _reflection.GeneratedProtocolMessageType('Peers', (_message.Message,), {
  'DESCRIPTOR' : _PEERS,
  '__module__' : 'diagnostics_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.diagnostics.Peers)
  })
_sym_db.RegisterMessage(Peers)

Peer = _reflection.GeneratedProtocolMessageType('Peer', (_message.Message,), {
  'DESCRIPTOR' : _PEER,
  '__module__' : 'diagnostics_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.diagnostics.Peer)
  })
_sym_db.RegisterMessage(Peer)



_DIAGNOSTICS = _descriptor.ServiceDescriptor(
  name='Diagnostics',
  full_name='io.casperlabs.node.api.diagnostics.Diagnostics',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=1496,
  serialized_end=2235,
  methods=[
  _descriptor.MethodDescriptor(
    name='ListPeers',
    full_name='io.casperlabs.node.api.diagnostics.Diagnostics.ListPeers',
    index=0,
    containing_service=None,
    input_type=empty__pb2._EMPTY,
    output_type=_PEERS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListDiscoveredPeers',
    full_name='io.casperlabs.node.api.diagnostics.Diagnostics.ListDiscoveredPeers',
    index=1,
    containing_service=None,
    input_type=empty__pb2._EMPTY,
    output_type=_PEERS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetProcessCpu',
    full_name='io.casperlabs.node.api.diagnostics.Diagnostics.GetProcessCpu',
    index=2,
    containing_service=None,
    input_type=empty__pb2._EMPTY,
    output_type=_PROCESSCPU,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetMemoryUsage',
    full_name='io.casperlabs.node.api.diagnostics.Diagnostics.GetMemoryUsage',
    index=3,
    containing_service=None,
    input_type=empty__pb2._EMPTY,
    output_type=_MEMORYUSAGE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetGarbageCollectors',
    full_name='io.casperlabs.node.api.diagnostics.Diagnostics.GetGarbageCollectors',
    index=4,
    containing_service=None,
    input_type=empty__pb2._EMPTY,
    output_type=_GARBAGECOLLECTORS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetMemoryPools',
    full_name='io.casperlabs.node.api.diagnostics.Diagnostics.GetMemoryPools',
    index=5,
    containing_service=None,
    input_type=empty__pb2._EMPTY,
    output_type=_MEMORYPOOLS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetThreads',
    full_name='io.casperlabs.node.api.diagnostics.Diagnostics.GetThreads',
    index=6,
    containing_service=None,
    input_type=empty__pb2._EMPTY,
    output_type=_THREADS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetNodeCoreMetrics',
    full_name='io.casperlabs.node.api.diagnostics.Diagnostics.GetNodeCoreMetrics',
    index=7,
    containing_service=None,
    input_type=empty__pb2._EMPTY,
    output_type=_NODECOREMETRICS,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_DIAGNOSTICS)

DESCRIPTOR.services_by_name['Diagnostics'] = _DIAGNOSTICS

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: casper.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import annotations_pb2 as annotations__pb2
from . import empty_pb2 as empty__pb2
from . import consensus_pb2 as consensus__pb2
from . import info_pb2 as info__pb2
from . import state_pb2 as state__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='casper.proto',
  package='io.casperlabs.node.api.casper',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x0c\x63\x61sper.proto\x12\x1dio.casperlabs.node.api.casper\x1a\x11\x61nnotations.proto\x1a\x0b\x65mpty.proto\x1a\x0f\x63onsensus.proto\x1a\ninfo.proto\x1a\x0bstate.proto\"G\n\rDeployRequest\x12\x36\n\x06\x64\x65ploy\x18\x01 \x01(\x0b\x32&.io.casperlabs.casper.consensus.Deploy\"s\n\x13GetBlockInfoRequest\x12\x19\n\x11\x62lock_hash_base16\x18\x01 \x01(\t\x12\x41\n\x04view\x18\x02 \x01(\x0e\x32\x33.io.casperlabs.casper.consensus.info.BlockInfo.View\"}\n\x17StreamBlockInfosRequest\x12\r\n\x05\x64\x65pth\x18\x01 \x01(\r\x12\x10\n\x08max_rank\x18\x02 \x01(\x04\x12\x41\n\x04view\x18\x03 \x01(\x0e\x32\x33.io.casperlabs.casper.consensus.info.BlockInfo.View\"v\n\x14GetDeployInfoRequest\x12\x1a\n\x12\x64\x65ploy_hash_base16\x18\x01 \x01(\t\x12\x42\n\x04view\x18\x02 \x01(\x0e\x32\x34.io.casperlabs.casper.consensus.info.DeployInfo.View\"z\n\x19StreamBlockDeploysRequest\x12\x19\n\x11\x62lock_hash_base16\x18\x01 \x01(\t\x12\x42\n\x04view\x18\x02 \x01(\x0e\x32\x34.io.casperlabs.casper.consensus.info.DeployInfo.View\"\xd9\x01\n\nStateQuery\x12I\n\x0bkey_variant\x18\x01 \x01(\x0e\x32\x34.io.casperlabs.node.api.casper.StateQuery.KeyVariant\x12\x12\n\nkey_base16\x18\x02 \x01(\t\x12\x15\n\rpath_segments\x18\x03 \x03(\t\"U\n\nKeyVariant\x12\x1b\n\x17KEY_VARIANT_UNSPECIFIED\x10\x00\x12\x08\n\x04HASH\x10\x01\x12\x08\n\x04UREF\x10\x02\x12\x0b\n\x07\x41\x44\x44RESS\x10\x03\x12\t\n\x05LOCAL\x10\x04\"k\n\x14GetBlockStateRequest\x12\x19\n\x11\x62lock_hash_base16\x18\x01 \x01(\t\x12\x38\n\x05query\x18\x02 \x01(\x0b\x32).io.casperlabs.node.api.casper.StateQuery\"r\n\x19\x42\x61tchGetBlockStateRequest\x12\x19\n\x11\x62lock_hash_base16\x18\x01 \x01(\t\x12:\n\x07queries\x18\x02 \x03(\x0b\x32).io.casperlabs.node.api.casper.StateQuery\"Y\n\x1a\x42\x61tchGetBlockStateResponse\x12;\n\x06values\x18\x01 \x03(\x0b\x32+.io.casperlabs.casper.consensus.state.Value2\xfb\x07\n\rCasperService\x12g\n\x06\x44\x65ploy\x12,.io.casperlabs.node.api.casper.DeployRequest\x1a\x16.google.protobuf.Empty\"\x17\x82\xd3\xe4\x93\x02\x11\"\x07\x64\x65ploys:\x06\x64\x65ploy\x12\x91\x01\n\x0cGetBlockInfo\x12\x32.io.casperlabs.node.api.casper.GetBlockInfoRequest\x1a..io.casperlabs.casper.consensus.info.BlockInfo\"\x1d\x82\xd3\xe4\x93\x02\x17\x12\x15{block_hash_base16=*}\x12\x8c\x01\n\x10StreamBlockInfos\x12\x36.io.casperlabs.node.api.casper.StreamBlockInfosRequest\x1a..io.casperlabs.casper.consensus.info.BlockInfo\"\x0e\x82\xd3\xe4\x93\x02\x08\x12\x06\x62locks0\x01\x12\x95\x01\n\rGetDeployInfo\x12\x33.io.casperlabs.node.api.casper.GetDeployInfoRequest\x1a/.io.casperlabs.casper.consensus.info.DeployInfo\"\x1e\x82\xd3\xe4\x93\x02\x18\x12\x16{deploy_hash_base16=*}\x12\x98\x01\n\x12StreamBlockDeploys\x12\x38.io.casperlabs.node.api.casper.StreamBlockDeploysRequest\x1a\x35.io.casperlabs.casper.consensus.Block.ProcessedDeploy\"\x0f\x82\xd3\xe4\x93\x02\t\x12\x07\x64\x65ploys0\x01\x12\x83\x01\n\rGetBlockState\x12\x33.io.casperlabs.node.api.casper.GetBlockStateRequest\x1a+.io.casperlabs.casper.consensus.state.Value\"\x10\x82\xd3\xe4\x93\x02\n\"\x05state:\x01*\x12\xa4\x01\n\x12\x42\x61tchGetBlockState\x12\x38.io.casperlabs.node.api.casper.BatchGetBlockStateRequest\x1a\x39.io.casperlabs.node.api.casper.BatchGetBlockStateResponse\"\x19\x82\xd3\xe4\x93\x02\x13\"\x0estate:batchGet:\x01*b\x06proto3')
  ,
  dependencies=[annotations__pb2.DESCRIPTOR,empty__pb2.DESCRIPTOR,consensus__pb2.DESCRIPTOR,info__pb2.DESCRIPTOR,state__pb2.DESCRIPTOR,])



_STATEQUERY_KEYVARIANT = _descriptor.EnumDescriptor(
  name='KeyVariant',
  full_name='io.casperlabs.node.api.casper.StateQuery.KeyVariant',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='KEY_VARIANT_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HASH', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UREF', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ADDRESS', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOCAL', index=4, number=4,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=815,
  serialized_end=900,
)
_sym_db.RegisterEnumDescriptor(_STATEQUERY_KEYVARIANT)


_DEPLOYREQUEST = _descriptor.Descriptor(
  name='DeployRequest',
  full_name='io.casperlabs.node.api.casper.DeployRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='deploy', full_name='io.casperlabs.node.api.casper.DeployRequest.deploy', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=121,
  serialized_end=192,
)


_GETBLOCKINFOREQUEST = _descriptor.Descriptor(
  name='GetBlockInfoRequest',
  full_name='io.casperlabs.node.api.casper.GetBlockInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_hash_base16', full_name='io.casperlabs.node.api.casper.GetBlockInfoRequest.block_hash_base16', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='view', full_name='io.casperlabs.node.api.casper.GetBlockInfoRequest.view', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=194,
  serialized_end=309,
)


_STREAMBLOCKINFOSREQUEST = _descriptor.Descriptor(
  name='StreamBlockInfosRequest',
  full_name='io.casperlabs.node.api.casper.StreamBlockInfosRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='depth', full_name='io.casperlabs.node.api.casper.StreamBlockInfosRequest.depth', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_rank', full_name='io.casperlabs.node.api.casper.StreamBlockInfosRequest.max_rank', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='view', full_name='io.casperlabs.node.api.casper.StreamBlockInfosRequest.view', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=311,
  serialized_end=436,
)


_GETDEPLOYINFOREQUEST = _descriptor.Descriptor(
  name='GetDeployInfoRequest',
  full_name='io.casperlabs.node.api.casper.GetDeployInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='deploy_hash_base16', full_name='io.casperlabs.node.api.casper.GetDeployInfoRequest.deploy_hash_base16', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='view', full_name='io.casperlabs.node.api.casper.GetDeployInfoRequest.view', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=438,
  serialized_end=556,
)


_STREAMBLOCKDEPLOYSREQUEST = _descriptor.Descriptor(
  name='StreamBlockDeploysRequest',
  full_name='io.casperlabs.node.api.casper.StreamBlockDeploysRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_hash_base16', full_name='io.casperlabs.node.api.casper.StreamBlockDeploysRequest.block_hash_base16', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='view', full_name='io.casperlabs.node.api.casper.StreamBlockDeploysRequest.view', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=558,
  serialized_end=680,
)


_STATEQUERY = _descriptor.Descriptor(
  name='StateQuery',
  full_name='io.casperlabs.node.api.casper.StateQuery',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key_variant', full_name='io.casperlabs.node.api.casper.StateQuery.key_variant', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='key_base16', full_name='io.casperlabs.node.api.casper.StateQuery.key_base16', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='path_segments', full_name='io.casperlabs.node.api.casper.StateQuery.path_segments', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _STATEQUERY_KEYVARIANT,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=683,
  serialized_end=900,
)


_GETBLOCKSTATEREQUEST = _descriptor.Descriptor(
  name='GetBlockStateRequest',
  full_name='io.casperlabs.node.api.casper.GetBlockStateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_hash_base16', full_name='io.casperlabs.node.api.casper.GetBlockStateRequest.block_hash_base16', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='query', full_name='io.casperlabs.node.api.casper.GetBlockStateRequest.query', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=902,
  serialized_end=1009,
)


_BATCHGETBLOCKSTATEREQUEST = _descriptor.Descriptor(
  name='BatchGetBlockStateRequest',
  full_name='io.casperlabs.node.api.casper.BatchGetBlockStateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_hash_base16', full_name='io.casperlabs.node.api.casper.BatchGetBlockStateRequest.block_hash_base16', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='queries', full_name='io.casperlabs.node.api.casper.BatchGetBlockStateRequest.queries', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1011,
  serialized_end=1125,
)


_BATCHGETBLOCKSTATERESPONSE = _descriptor.Descriptor(
  name='BatchGetBlockStateResponse',
  full_name='io.casperlabs.node.api.casper.BatchGetBlockStateResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='io.casperlabs.node.api.casper.BatchGetBlockStateResponse.values', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1127,
  serialized_end=1216,
)

_DEPLOYREQUEST.fields_by_name['deploy'].message_type = consensus__pb2._DEPLOY
_GETBLOCKINFOREQUEST.fields_by_name['view'].enum_type = info__pb2._BLOCKINFO_VIEW
_STREAMBLOCKINFOSREQUEST.fields_by_name['view'].enum_type = info__pb2._BLOCKINFO_VIEW
_GETDEPLOYINFOREQUEST.fields_by_name['view'].enum_type = info__pb2._DEPLOYINFO_VIEW
_STREAMBLOCKDEPLOYSREQUEST.fields_by_name['view'].enum_type = info__pb2._DEPLOYINFO_VIEW
_STATEQUERY.fields_by_name['key_variant'].enum_type = _STATEQUERY_KEYVARIANT
_STATEQUERY_KEYVARIANT.containing_type = _STATEQUERY
_GETBLOCKSTATEREQUEST.fields_by_name['query'].message_type = _STATEQUERY
_BATCHGETBLOCKSTATEREQUEST.fields_by_name['queries'].message_type = _STATEQUERY
_BATCHGETBLOCKSTATERESPONSE.fields_by_name['values'].message_type = state__pb2._VALUE
DESCRIPTOR.message_types_by_name['DeployRequest'] = _DEPLOYREQUEST
DESCRIPTOR.message_types_by_name['GetBlockInfoRequest'] = _GETBLOCKINFOREQUEST
DESCRIPTOR.message_types_by_name['StreamBlockInfosRequest'] = _STREAMBLOCKINFOSREQUEST
DESCRIPTOR.message_types_by_name['GetDeployInfoRequest'] = _GETDEPLOYINFOREQUEST
DESCRIPTOR.message_types_by_name['StreamBlockDeploysRequest'] = _STREAMBLOCKDEPLOYSREQUEST
DESCRIPTOR.message_types_by_name['StateQuery'] = _STATEQUERY
DESCRIPTOR.message_types_by_name['GetBlockStateRequest'] = _GETBLOCKSTATEREQUEST
DESCRIPTOR.message_types_by_name['BatchGetBlockStateRequest'] = _BATCHGETBLOCKSTATEREQUEST
DESCRIPTOR.message_types_by_name['BatchGetBlockStateResponse'] = _BATCHGETBLOCKSTATERESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DeployRequest = _reflection.GeneratedProtocolMessageType('DeployRequest', (_message.Message,), {
  'DESCRIPTOR' : _DEPLOYREQUEST,
  '__module__' : 'casper_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.casper.DeployRequest)
  })
_sym_db.RegisterMessage(DeployRequest)

GetBlockInfoRequest = _reflection.GeneratedProtocolMessageType('GetBlockInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETBLOCKINFOREQUEST,
  '__module__' : 'casper_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.casper.GetBlockInfoRequest)
  })
_sym_db.RegisterMessage(GetBlockInfoRequest)

StreamBlockInfosRequest = _reflection.GeneratedProtocolMessageType('StreamBlockInfosRequest', (_message.Message,), {
  'DESCRIPTOR' : _STREAMBLOCKINFOSREQUEST,
  '__module__' : 'casper_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.casper.StreamBlockInfosRequest)
  })
_sym_db.RegisterMessage(StreamBlockInfosRequest)

GetDeployInfoRequest = _reflection.GeneratedProtocolMessageType('GetDeployInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETDEPLOYINFOREQUEST,
  '__module__' : 'casper_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.casper.GetDeployInfoRequest)
  })
_sym_db.RegisterMessage(GetDeployInfoRequest)

StreamBlockDeploysRequest = _reflection.GeneratedProtocolMessageType('StreamBlockDeploysRequest', (_message.Message,), {
  'DESCRIPTOR' : _STREAMBLOCKDEPLOYSREQUEST,
  '__module__' : 'casper_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.casper.StreamBlockDeploysRequest)
  })
_sym_db.RegisterMessage(StreamBlockDeploysRequest)

StateQuery = _reflection.GeneratedProtocolMessageType('StateQuery', (_message.Message,), {
  'DESCRIPTOR' : _STATEQUERY,
  '__module__' : 'casper_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.casper.StateQuery)
  })
_sym_db.RegisterMessage(StateQuery)

GetBlockStateRequest = _reflection.GeneratedProtocolMessageType('GetBlockStateRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETBLOCKSTATEREQUEST,
  '__module__' : 'casper_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.casper.GetBlockStateRequest)
  })
_sym_db.RegisterMessage(GetBlockStateRequest)

BatchGetBlockStateRequest = _reflection.GeneratedProtocolMessageType('BatchGetBlockStateRequest', (_message.Message,), {
  'DESCRIPTOR' : _BATCHGETBLOCKSTATEREQUEST,
  '__module__' : 'casper_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.casper.BatchGetBlockStateRequest)
  })
_sym_db.RegisterMessage(BatchGetBlockStateRequest)

BatchGetBlockStateResponse = _reflection.GeneratedProtocolMessageType('BatchGetBlockStateResponse', (_message.Message,), {
  'DESCRIPTOR' : _BATCHGETBLOCKSTATERESPONSE,
  '__module__' : 'casper_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.casper.BatchGetBlockStateResponse)
  })
_sym_db.RegisterMessage(BatchGetBlockStateResponse)



_CASPERSERVICE = _descriptor.ServiceDescriptor(
  name='CasperService',
  full_name='io.casperlabs.node.api.casper.CasperService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=1219,
  serialized_end=2238,
  methods=[
  _descriptor.MethodDescriptor(
    name='Deploy',
    full_name='io.casperlabs.node.api.casper.CasperService.Deploy',
    index=0,
    containing_service=None,
    input_type=_DEPLOYREQUEST,
    output_type=empty__pb2._EMPTY,
    serialized_options=_b('\202\323\344\223\002\021\"\007deploys:\006deploy'),
  ),
  _descriptor.MethodDescriptor(
    name='GetBlockInfo',
    full_name='io.casperlabs.node.api.casper.CasperService.GetBlockInfo',
    index=1,
    containing_service=None,
    input_type=_GETBLOCKINFOREQUEST,
    output_type=info__pb2._BLOCKINFO,
    serialized_options=_b('\202\323\344\223\002\027\022\025{block_hash_base16=*}'),
  ),
  _descriptor.MethodDescriptor(
    name='StreamBlockInfos',
    full_name='io.casperlabs.node.api.casper.CasperService.StreamBlockInfos',
    index=2,
    containing_service=None,
    input_type=_STREAMBLOCKINFOSREQUEST,
    output_type=info__pb2._BLOCKINFO,
    serialized_options=_b('\202\323\344\223\002\010\022\006blocks'),
  ),
  _descriptor.MethodDescriptor(
    name='GetDeployInfo',
    full_name='io.casperlabs.node.api.casper.CasperService.GetDeployInfo',
    index=3,
    containing_service=None,
    input_type=_GETDEPLOYINFOREQUEST,
    output_type=info__pb2._DEPLOYINFO,
    serialized_options=_b('\202\323\344\223\002\030\022\026{deploy_hash_base16=*}'),
  ),
  _descriptor.MethodDescriptor(
    name='StreamBlockDeploys',
    full_name='io.casperlabs.node.api.casper.CasperService.StreamBlockDeploys',
    index=4,
    containing_service=None,
    input_type=_STREAMBLOCKDEPLOYSREQUEST,
    output_type=consensus__pb2._BLOCK_PROCESSEDDEPLOY,
    serialized_options=_b('\202\323\344\223\002\t\022\007deploys'),
  ),
  _descriptor.MethodDescriptor(
    name='GetBlockState',
    full_name='io.casperlabs.node.api.casper.CasperService.GetBlockState',
    index=5,
    containing_service=None,
    input_type=_GETBLOCKSTATEREQUEST,
    output_type=state__pb2._VALUE,
    serialized_options=_b('\202\323\344\223\002\n\"\005state:\001*'),
  ),
  _descriptor.MethodDescriptor(
    name='BatchGetBlockState',
    full_name='io.casperlabs.node.api.casper.CasperService.BatchGetBlockState',
    index=6,
    containing_service=None,
    input_type=_BATCHGETBLOCKSTATEREQUEST,
    output_type=_BATCHGETBLOCKSTATERESPONSE,
    serialized_options=_b('\202\323\344\223\002\023\"\016state:batchGet:\001*'),
  ),
])
_sym_db.RegisterServiceDescriptor(_CASPERSERVICE)

DESCRIPTOR.services_by_name['CasperService'] = _CASPERSERVICE

# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3
"""
CasperLabs Client API library and command line tool.
"""

# Hack to fix the relative imports problems #
import sys
from pathlib import Path

file = Path(__file__).resolve()
parent, root = file.parent, file.parents[1]
sys.path.append(str(root))

# end of hack #
import os
import time
import argparse
import grpc
from grpc._channel import _Rendezvous
import ssl
import functools
from pyblake2 import blake2b
import ed25519
import base64
import json
import struct
import logging
import pkg_resources

# Monkey patching of google.protobuf.text_encoding.CEscape
# to get keys and signatures in hex when printed
import google.protobuf.text_format

CEscape = google.protobuf.text_format.text_encoding.CEscape

base64_b64decode = base64.b64decode


def _hex(text, as_utf8):
    try:
        return (len(text) in (32, 64, 20)) and text.hex() or CEscape(text, as_utf8)
    except TypeError:
        return CEscape(text, as_utf8)


google.protobuf.text_format.text_encoding.CEscape = _hex

# ~/CasperLabs/protobuf/io/casperlabs/node/api/control.proto
from .control_pb2_grpc import ControlServiceStub
from . import control_pb2 as control

# ~/CasperLabs/protobuf/io/casperlabs/node/api/casper.proto
from . import casper_pb2 as casper
from .casper_pb2_grpc import CasperServiceStub

# ~/CasperLabs/protobuf/io/casperlabs/casper/consensus/consensus.proto
from . import consensus_pb2 as consensus, state_pb2 as state

# ~/CasperLabs/protobuf/io/casperlabs/casper/consensus/info.proto
from . import info_pb2 as info

DEFAULT_HOST = "127.0.0.1"
DEFAULT_PORT = 40401
DEFAULT_INTERNAL_PORT = 40402


from google.protobuf import json_format

Arg = consensus.Deploy.Arg
Value = consensus.Deploy.Arg.Value


class ABI:
    """
    Serialize deploy arguments.
    """

    @staticmethod
    def optional_value(name, a):
        if a is None:
            return Arg(name=name, value=Value(optional_value=Value()))
        return Arg(name=name, value=Value(optional_value=a.value))

    @staticmethod
    def bytes_value(name, a: bytes):
        return Arg(name=name, value=Value(bytes_value=a))

    @staticmethod
    def account(name, a):
        if type(a) == bytes and len(a) == 32:
            return ABI.byte_array(name, a)
        if type(a) == str and len(a) == 64:
            return ABI.byte_array(name, bytes.fromhex(a))
        raise Exception("account must be 32 bytes or 64 characters long string")

    @staticmethod
    def int_value(name, a: int):
        return Arg(name=name, value=Value(int_value=a))

    @staticmethod
    def long_value(name, a: int):
        return Arg(name=name, value=Value(long_value=a))

    @staticmethod
    def big_int(name, a):
        return Arg(
            name=name, value=Value(big_int=state.BigInt(value=str(a), bit_width=512))
        )

    @staticmethod
    def string_value(name, a):
        return Arg(name=name, value=Value(string_value=a))

    @staticmethod
    def args(l: list):
        c = consensus.Deploy.Code(args=l)
        return c.args

    @staticmethod
    def args_from_json(s):
        base64_b64decode = base64.b64decode
        try:
            # Change JSON protobuf format of binary data from base64 to base16
            base64.b64decode = lambda s: bytes.fromhex(s)
            parsed_json = json.loads(s)
            args = [
                json_format.ParseDict(d, consensus.Deploy.Arg()) for d in parsed_json
            ]
            c = consensus.Deploy.Code(args=args)
            return c.args
        finally:
            base64.b64decode = base64_b64decode

    @staticmethod
    def args_to_json(args):
        base64_b64encode = base64.b64encode
        try:
            # We can't just call MessageToDict or MessageToJson on the args object,
            # which is a 'repeated Arg', because we get:
            # AttributeError: 'google.protobuf.pyext._message.RepeatedCompositeCo' object has no attribute 'DESCRIPTOR'
            class Mock:
                def __init__(self, v):
                    self.value = v

                def decode(self, s):
                    return self.value

            base64.b64encode = lambda b: Mock(b.hex())
            return json.dumps(
                [
                    json_format.MessageToDict(arg, preserving_proto_field_name=True)
                    for arg in args
                ]
            )
        finally:
            base64.b64encode = base64_b64encode

    # Below methods for backwards compatibility

    @staticmethod
    def u32(name, n: int):
        return ABI.int_value(name, n)

    @staticmethod
    def u64(name, n: int):
        return ABI.long_value(name, n)

    @staticmethod
    def u512(name, n: int):
        return ABI.big_int(name, n)

    @staticmethod
    def byte_array(name, a):
        return Arg(name=name, value=Value(bytes_value=a))


def read_pem_key(file_name: str):
    with open(file_name) as f:
        s = [l for l in f.readlines() if l and not l.startswith("-----")][0].strip()
        r = base64_b64decode(s)
        return len(r) % 32 == 0 and r[:32] or r[-32:]


class InternalError(Exception):
    """
    The only exception that API calls can throw.
    Internal errors like gRPC exceptions will be caught
    and this exception thrown instead, so the user does
    not have to worry about handling any other exceptions.
    """

    def __init__(self, status="", details=""):
        super(InternalError, self).__init__()
        self.status = status
        self.details = details

    def __str__(self):
        return f"{self.status}: {self.details}"


def api(function):
    """
    Decorator of API functions that protects user code from
    unknown exceptions raised by gRPC or internal API errors.
    It will catch all exceptions and throw InternalError.

    :param function: function to be decorated
    :return:
    """

    @functools.wraps(function)
    def wrapper(*args, **kwargs):
        try:
            return function(*args, **kwargs)
        except (SyntaxError, TypeError, InternalError):
            raise
        except _Rendezvous as e:
            raise InternalError(str(e.code()), e.details())
        except Exception as e:
            raise InternalError(details=str(e)) from e

    return wrapper


def blake2b_hash(data: bytes) -> bytes:
    h = blake2b(digest_size=32)
    h.update(data)
    return h.digest()


def _read_binary(file_name: str):
    with open(file_name, "rb") as f:
        return f.read()


def _encode_contract(contract_options, contract_args):
    """
    """
    file_name, hash, name, uref = contract_options
    C = consensus.Deploy.Code
    if file_name:
        return C(wasm=_read_binary(file_name), args=contract_args)
    if hash:
        return C(hash=hash, args=contract_args)
    if name:
        return C(name=name, args=contract_args)
    if uref:
        return C(uref=uref, args=contract_args)
    raise Exception("One of wasm, hash, name or uref is required")


def signature(private_key, data: bytes):
    return private_key and consensus.Signature(
        sig_algorithm="ed25519",
        sig=ed25519.SigningKey(read_pem_key(private_key)).sign(data),
    )


def _serialize(o) -> bytes:
    return o.SerializeToString()


NUMBER_OF_RETRIES = 5

# Initial delay in seconds before an attempt to retry
INITIAL_DELAY = 0.3


def retry_wrapper(function, *args):
    delay = INITIAL_DELAY
    for i in range(NUMBER_OF_RETRIES):
        try:
            return function(*args)
        except _Rendezvous as e:
            if e.code() == grpc.StatusCode.UNAVAILABLE and i < NUMBER_OF_RETRIES - 1:
                delay += delay
                logging.warning(f"Retrying after {e} in {delay} seconds")
                time.sleep(delay)
            else:
                raise


def retry_unary(function):
    @functools.wraps(function)
    def wrapper(*args):
        return retry_wrapper(function, *args)

    return wrapper


def retry_stream(function):
    @functools.wraps(function)
    def wrapper(*args):
        yield from retry_wrapper(function, *args)

    return wrapper


class InsecureGRPCService:
    def __init__(self, host, port, serviceStub):
        self.address = f"{host}:{port}"
        self.serviceStub = serviceStub

    def __getattr__(self, name):

        logging.warning(
            f"Creating insecure connection to {self.address} ({self.serviceStub})"
        )

        @retry_unary
        def unary_unary(*args):
            logging.debug(
                f"Insecure {self.address} ({self.serviceStub}): {name} {list(args)}"
            )
            with grpc.insecure_channel(self.address) as channel:
                return getattr(self.serviceStub(channel), name)(*args)

        @retry_stream
        def unary_stream(*args):
            logging.debug(
                f"Insecure {self.address} ({self.serviceStub}): {name} {list(args)}"
            )
            with grpc.insecure_channel(self.address) as channel:
                yield from getattr(self.serviceStub(channel), name[: -len("_stream")])(
                    *args
                )

        return name.endswith("_stream") and unary_stream or unary_unary


def extract_common_name(certificate_file: str) -> str:
    cert_dict = ssl._ssl._test_decode_cert(certificate_file)
    return [t[0][1] for t in cert_dict["subject"] if t[0][0] == "commonName"][0]


class SecureGRPCService:
    def __init__(self, host, port, serviceStub, node_id, certificate_file):
        self.address = f"{host}:{port}"
        self.serviceStub = serviceStub
        self.node_id = node_id or extract_common_name(certificate_file)
        self.certificate_file = certificate_file
        with open(self.certificate_file, "rb") as f:
            self.credentials = grpc.ssl_channel_credentials(f.read())
        self.secure_channel_options = self.node_id and (
            ("grpc.ssl_target_name_override", self.node_id),
            ("grpc.default_authority", self.node_id),
        )

    def __getattr__(self, name):
        logging.debug(
            f"Creating secure connection to {self.address} ({self.serviceStub})"
        )

        @retry_unary
        def unary_unary(*args):
            with grpc.secure_channel(
                self.address, self.credentials, options=self.secure_channel_options
            ) as channel:
                return getattr(self.serviceStub(channel), name)(*args)

        @retry_stream
        def unary_stream(*args):
            with grpc.secure_channel(
                self.address, self.credentials, options=self.secure_channel_options
            ) as channel:
                yield from getattr(self.serviceStub(channel), name[: -len("_stream")])(
                    *args
                )

        return name.endswith("_stream") and unary_stream or unary_unary


class CasperLabsClient:
    """
    gRPC CasperLabs client.
    """

    # Note, there is also casper.StateQuery.KeyVariant.KEY_VARIANT_UNSPECIFIED,
    # but it doesn't seem to have an official string representation
    # ("key_variant_unspecified"? "unspecified"?) and is not used by the client.
    STATE_QUERY_KEY_VARIANT = {
        "hash": casper.StateQuery.KeyVariant.HASH,
        "uref": casper.StateQuery.KeyVariant.UREF,
        "address": casper.StateQuery.KeyVariant.ADDRESS,
        "local": casper.StateQuery.KeyVariant.LOCAL,
    }

    def __init__(
        self,
        host: str = DEFAULT_HOST,
        port: int = DEFAULT_PORT,
        port_internal: int = DEFAULT_INTERNAL_PORT,
        node_id: str = None,
        certificate_file: str = None,
    ):
        """
        CasperLabs client's constructor.

        :param host:            Hostname or IP of node on which gRPC service is running
        :param port:            Port used for external gRPC API
        :param port_internal:   Port used for internal gRPC API
        :param certificate_file:      Certificate file for TLS
        :param node_id:         node_id of the node, for gRPC encryption
        """
        self.host = host
        self.port = port
        self.port_internal = port_internal
        self.node_id = node_id
        self.certificate_file = certificate_file

        if node_id:
            self.casperService = SecureGRPCService(
                host, port, CasperServiceStub, node_id, certificate_file
            )
            self.controlService = SecureGRPCService(
                # We currently assume that if node_id is given then
                # we get certificate_file too. This is unlike in the Scala client
                # where node_id is all that's needed for configuring secure connection.
                # The reason for this is that currently it doesn't seem to be possible
                # to open a secure grpc connection in Python without supplying any
                # certificate on the client side.
                host,
                port_internal,
                ControlServiceStub,
                node_id,
                certificate_file,
            )
        else:
            self.casperService = InsecureGRPCService(host, port, CasperServiceStub)
            self.controlService = InsecureGRPCService(
                host, port_internal, ControlServiceStub
            )

    @api
    def make_deploy(
        self,
        from_addr: bytes = None,
        gas_price: int = 10,
        payment: str = None,
        session: str = None,
        public_key: str = None,
        session_args: bytes = None,
        payment_args: bytes = None,
        payment_hash: bytes = None,
        payment_name: str = None,
        payment_uref: bytes = None,
        session_hash: bytes = None,
        session_name: str = None,
        session_uref: bytes = None,
        ttl_millis: int = 0,
        dependencies: list = None,
    ):
        """
        Create a protobuf deploy object. See deploy for description of parameters.
        """
        # Convert from hex to binary.
        if from_addr and len(from_addr) == 64:
            from_addr = bytes.fromhex(from_addr)

        if from_addr and len(from_addr) != 32:
            raise Exception(f"from_addr must be 32 bytes")

        session_options = (session, session_hash, session_name, session_uref)
        payment_options = (payment, payment_hash, payment_name, payment_uref)

        # Compatibility mode, should be removed when payment is obligatory
        if len(list(filter(None, payment_options))) == 0:
            logging.info("No payment contract provided, using session as payment")
            payment_options = session_options

        if len(list(filter(None, session_options))) != 1:
            raise TypeError(
                "deploy: only one of session, session_hash, session_name, session_uref must be provided"
            )

        if len(list(filter(None, payment_options))) != 1:
            raise TypeError(
                "deploy: only one of payment, payment_hash, payment_name, payment_uref must be provided"
            )

        # session_args must go to payment as well for now cause otherwise we'll get GASLIMIT error,
        # if payment is same as session:
        # https://github.com/CasperLabs/CasperLabs/blob/dev/casper/src/main/scala/io/casperlabs/casper/util/ProtoUtil.scala#L463
        body = consensus.Deploy.Body(
            session=_encode_contract(session_options, session_args),
            payment=_encode_contract(payment_options, payment_args),
        )

        header = consensus.Deploy.Header(
            account_public_key=from_addr or (public_key and read_pem_key(public_key)),
            timestamp=int(1000 * time.time()),
            gas_price=gas_price,
            body_hash=blake2b_hash(_serialize(body)),
            ttl_millis=ttl_millis,
            dependencies=dependencies
            and [bytes.fromhex(d) for d in dependencies]
            or [],
        )

        deploy_hash = blake2b_hash(_serialize(header))

        return consensus.Deploy(deploy_hash=deploy_hash, header=header, body=body)

    @api
    def sign_deploy(self, deploy, public_key, private_key_file):
        deploy.approvals.extend(
            [
                consensus.Approval(
                    approver_public_key=public_key,
                    signature=signature(private_key_file, deploy.deploy_hash),
                )
            ]
        )
        return deploy

    @api
    def deploy(
        self,
        from_addr: bytes = None,
        gas_price: int = 10,
        payment: str = None,
        session: str = None,
        public_key: str = None,
        private_key: str = None,
        session_args: bytes = None,
        payment_args: bytes = None,
        payment_hash: bytes = None,
        payment_name: str = None,
        payment_uref: bytes = None,
        session_hash: bytes = None,
        session_name: str = None,
        session_uref: bytes = None,
        ttl_millis: int = 0,
        dependencies=None,
    ):
        """
        Deploy a smart contract source file to Casper on an existing running node.
        The deploy will be packaged and sent as a block to the network depending
        on the configuration of the Casper instance.

        :param from_addr:     Purse address that will be used to pay for the deployment.
        :param gas_price:     The price of gas for this transaction in units dust/gas.
                              Must be positive integer.
        :param payment:       Path to the file with payment code.
        :param session:       Path to the file with session code.
        :param public_key:    Path to a file with public key (Ed25519)
        :param private_key:   Path to a file with private key (Ed25519)
        :param session_args:  List of ABI encoded arguments of session contract
        :param payment_args:  List of ABI encoded arguments of payment contract
        :param session-hash:  Hash of the stored contract to be called in the
                              session; base16 encoded.
        :param session-name:  Name of the stored contract (associated with the
                              executing account) to be called in the session.
        :param session-uref:  URef of the stored contract to be called in the
                              session; base16 encoded.
        :param payment-hash:  Hash of the stored contract to be called in the
                              payment; base16 encoded.
        :param payment-name:  Name of the stored contract (associated with the
                              executing account) to be called in the payment.
        :param payment-uref:  URef of the stored contract to be called in the
                              payment; base16 encoded.
        :ttl_millis:          Time to live. Time (in milliseconds) that the
                              deploy will remain valid for.
        :dependencies:        List of deploy hashes (base16 encoded) which
                              must be executed before this deploy.
        :return:              Tuple: (deserialized DeployServiceResponse object, deploy_hash)
        """

        deploy = self.make_deploy(
            from_addr=from_addr,
            gas_price=gas_price,
            payment=payment,
            session=session,
            public_key=public_key,
            session_args=session_args,
            payment_args=payment_args,
            payment_hash=payment_hash,
            payment_name=payment_name,
            payment_uref=payment_uref,
            session_hash=session_hash,
            session_name=session_name,
            session_uref=session_uref,
            ttl_millis=ttl_millis,
            dependencies=dependencies,
        )

        deploy = self.sign_deploy(
            deploy, (public_key and read_pem_key(public_key)) or from_addr, private_key
        )

        # TODO: Return only deploy_hash
        return self.send_deploy(deploy), deploy.deploy_hash

    @api
    def send_deploy(self, deploy):
        # TODO: Deploy returns Empty, error handing via exceptions, apparently,
        # so no point in returning it.
        return self.casperService.Deploy(casper.DeployRequest(deploy=deploy))

    @api
    def showBlocks(self, depth: int = 1, max_rank=0, full_view=True):
        """
        Get slices of the DAG, going backwards, rank by rank.

        :param depth:     How many of the top ranks of the DAG to show.
        :param max_rank:  Maximum rank to go back from.
                          0 means go from the current tip of the DAG.
        :param full_view: Full view if True, otherwise basic.
        :return:          Generator of block info objects.
        """
        yield from self.casperService.StreamBlockInfos_stream(
            casper.StreamBlockInfosRequest(
                depth=depth,
                max_rank=max_rank,
                view=(
                    full_view and info.BlockInfo.View.FULL or info.BlockInfo.View.BASIC
                ),
            )
        )

    @api
    def showBlock(self, block_hash_base16: str, full_view=True):
        """
        Returns object describing a block known by Casper on an existing running node.

        :param block_hash_base16: hash of the block to be retrieved
        :param full_view:         full view if True, otherwise basic
        :return:                  object representing the retrieved block
        """
        return self.casperService.GetBlockInfo(
            casper.GetBlockInfoRequest(
                block_hash_base16=block_hash_base16,
                view=(
                    full_view and info.BlockInfo.View.FULL or info.BlockInfo.View.BASIC
                ),
            )
        )

    @api
    def propose(self):
        """"
        Propose a block using deploys in the pool.

        :return:    response object with block_hash
        """
        return self.controlService.Propose(control.ProposeRequest())

    @api
    def visualizeDag(
        self,
        depth: int,
        out: str = None,
        show_justification_lines: bool = False,
        stream: str = None,
    ):
        """
        Retrieve DAG in DOT format.

        :param depth:                     depth in terms of block height
        :param out:                       output image filename, outputs to stdout if
                                          not specified, must end with one of the png,
                                          svg, svg_standalone, xdot, plain, plain_ext,
                                          ps, ps2, json, json0
        :param show_justification_lines:  if justification lines should be shown
        :param stream:                    subscribe to changes, 'out' has to specified,
                                          valid values are 'single-output', 'multiple-outputs'
        :return:                          VisualizeBlocksResponse object
        """
        raise Exception("Not implemented yet")

    @api
    def queryState(self, blockHash: str, key: str, path: str, keyType: str):
        """
        Query a value in the global state.

        :param blockHash:         Hash of the block to query the state of
        :param key:               Base16 encoding of the base key
        :param path:              Path to the value to query. Must be of the form
                                  'key1/key2/.../keyn'
        :param keyType:           Type of base key. Must be one of 'hash', 'uref', 'address' or 'local'.
                                  For 'local' key type, 'key' value format is {seed}:{rest},
                                  where both parts are hex encoded."
        :return:                  QueryStateResponse object
        """

        def key_variant(keyType):

            variant = self.STATE_QUERY_KEY_VARIANT.get(keyType.lower(), None)
            if variant is None:
                raise InternalError(
                    "query-state", f"{keyType} is not a known query-state key type"
                )
            return variant

        q = casper.StateQuery(key_variant=key_variant(keyType), key_base16=key)
        q.path_segments.extend([name for name in path.split("/") if name])
        return self.casperService.GetBlockState(
            casper.GetBlockStateRequest(block_hash_base16=blockHash, query=q)
        )

    @api
    def balance(self, address: str, block_hash: str):
        value = self.queryState(block_hash, address, "", "address")
        account = None
        try:
            account = value.account
        except AttributeError:
            return InternalError(
                "balance", f"Expected Account type value under {address}."
            )

        urefs = [u for u in account.named_keys if u.name == "mint"]
        if len(urefs) == 0:
            raise InternalError(
                "balance",
                "Account's named_keys map did not contain Mint contract address.",
            )

        mintPublic = urefs[0]

        def abi_byte_array(a: bytes) -> bytes:
            return struct.pack("<I", len(a)) + a

        mintPublicHex = mintPublic.key.uref.uref.hex()
        purseAddrHex = abi_byte_array(account.purse_id.uref).hex()
        localKeyValue = f"{mintPublicHex}:{purseAddrHex}"

        balanceURef = self.queryState(block_hash, localKeyValue, "", "local")
        balance = self.queryState(
            block_hash, balanceURef.key.uref.uref.hex(), "", "uref"
        )
        return int(balance.big_int.value)

    @api
    def showDeploy(self, deploy_hash_base16: str, full_view=True):
        """
        Retrieve information about a single deploy by hash.
        """
        return self.casperService.GetDeployInfo(
            casper.GetDeployInfoRequest(
                deploy_hash_base16=deploy_hash_base16,
                view=(
                    full_view
                    and info.DeployInfo.View.FULL
                    or info.DeployInfo.View.BASIC
                ),
            )
        )

    @api
    def showDeploys(self, block_hash_base16: str, full_view=True):
        """
        Get the processed deploys within a block.
        """
        yield from self.casperService.StreamBlockDeploys_stream(
            casper.StreamBlockDeploysRequest(
                block_hash_base16=block_hash_base16,
                view=(
                    full_view
                    and info.DeployInfo.View.FULL
                    or info.DeployInfo.View.BASIC
                ),
            )
        )


def guarded_command(function):
    """
    Decorator of functions that implement CLI commands.

    Occasionally the node can throw some exceptions instead of properly sending us a response,
    those will be deserialized on our end and rethrown by the gRPC layer.
    In this case we want to catch the exception and return a non-zero return code to the shell.

    :param function:  function to be decorated
    :return:
    """

    @functools.wraps(function)
    def wrapper(*args, **kwargs):
        try:
            rc = function(*args, **kwargs)
            # Generally the CLI commands are assumed to succeed if they don't throw,
            # but they can also return a positive error code if they need to.
            if rc is not None:
                return rc
            return 0
        except Exception as e:
            print(str(e), file=sys.stderr)
            return 1

    return wrapper


def hexify(o):
    """
    Convert protobuf message to text format with cryptographic keys and signatures in base 16.
    """
    return google.protobuf.text_format.MessageToString(o)


def _show_blocks(response, element_name="block"):
    count = 0
    for block in response:
        print(f"------------- {element_name} {count} ---------------")
        print(hexify(block))
        print("-----------------------------------------------------\n")
        count += 1
    print("count:", count)


def _show_block(response):
    print(hexify(response))


def bundled_contract(file_name):
    """
    Return path to contract file bundled with the package.
    """
    p = pkg_resources.resource_filename(__name__, file_name)
    if not os.path.exists(p):
        raise Exception(f"Missing bundled contract {file_name} ({p})")
    return p


def _set_session(args, file_name):
    """
    Use bundled contract unless one of the session* args is set.
    """
    if not any((args.session, args.session_hash, args.session_name, args.session_uref)):
        args.session = bundled_contract(file_name)


@guarded_command
def no_command(casperlabs_client, args):
    print("You must provide a command. --help for documentation of commands.")
    return 1


@guarded_command
def bond_command(casperlabs_client, args):
    logging.info(f"BOND {args}")
    _set_session(args, "bonding.wasm")

    if not args.session_args:
        args.session_args = ABI.args_to_json(
            ABI.args([ABI.long_value("amount", args.amount)])
        )

    return deploy_command(casperlabs_client, args)


@guarded_command
def unbond_command(casperlabs_client, args):
    logging.info(f"UNBOND {args}")
    _set_session(args, "unbonding.wasm")

    if not args.session_args:
        args.session_args = ABI.args_to_json(
            ABI.args(
                [ABI.optional_value("amount", ABI.long_value("amount", args.amount))]
            )
        )

    logging.info(f" XXX unbond_command: args.session_args={args.session_args}")

    return deploy_command(casperlabs_client, args)


def _deploy_kwargs(args, private_key_accepted=True):
    from_addr = bytes.fromhex(getattr(args, "from"))
    if len(from_addr) != 32:
        raise Exception(
            "--from must be 32 bytes encoded as 64 characters long hexadecimal"
        )

    if args.payment_amount:
        args.payment_args = ABI.args_to_json(
            ABI.args([ABI.big_int("amount", int(args.payment_amount))])
        )
        # Unless one of payment* options supplied use bundled standard-payment
        if not any(
            (args.payment, args.payment_name, args.payment_hash, args.payment_uref)
        ):
            args.payment = bundled_contract("standard_payment.wasm")

    d = dict(
        from_addr=from_addr,
        gas_price=args.gas_price,
        payment=args.payment or args.session,
        session=args.session,
        public_key=args.public_key or None,
        session_args=args.session_args
        and ABI.args_from_json(args.session_args)
        or None,
        payment_args=args.payment_args
        and ABI.args_from_json(args.payment_args)
        or None,
        payment_hash=args.payment_hash and bytes.fromhex(args.payment_hash),
        payment_name=args.payment_name,
        payment_uref=args.payment_uref and bytes.fromhex(args.payment_uref),
        session_hash=args.session_hash and bytes.fromhex(args.session_hash),
        session_name=args.session_name,
        session_uref=args.session_uref and bytes.fromhex(args.session_uref),
        ttl_millis=args.ttl,
        dependencies=args.dependencies,
    )
    if private_key_accepted:
        d["private_key"] = args.private_key or None
    return d


@guarded_command
def make_deploy_command(casperlabs_client, args):
    kwargs = _deploy_kwargs(args, private_key_accepted=False)
    deploy = casperlabs_client.make_deploy(**kwargs)
    data = deploy.SerializeToString()
    if not args.deploy_path:
        sys.stdout.write(data)
    else:
        with open(args.deploy_path, "wb") as f:
            f.write(data)


@guarded_command
def sign_deploy_command(casperlabs_client, args):
    deploy = consensus.Deploy()
    if args.deploy_path:
        with open(args.deploy_path, "rb") as input_file:
            deploy.ParseFromString(input_file.read())
    else:
        deploy.ParseFromString(sys.stdin.read())

    deploy = casperlabs_client.sign_deploy(
        deploy, read_pem_key(args.public_key), args.private_key
    )

    if not args.signed_deploy_path:
        sys.stdout.write(deploy.SerializeToString())
    else:
        with open(args.signed_deploy_path, "wb") as output_file:
            output_file.write(deploy.SerializeToString())


@guarded_command
def send_deploy_command(casperlabs_client, args):
    deploy = consensus.Deploy()
    with open(args.deploy_path, "rb") as f:
        deploy.ParseFromString(f.read())
        casperlabs_client.send_deploy(deploy)
    print(f"Success! Deploy {deploy.deploy_hash.hex()} deployed")


@guarded_command
def deploy_command(casperlabs_client, args):
    kwargs = _deploy_kwargs(args)
    _, deploy_hash = casperlabs_client.deploy(**kwargs)
    print(f"Success! Deploy {deploy_hash.hex()} deployed")


@guarded_command
def propose_command(casperlabs_client, args):
    response = casperlabs_client.propose()
    print(f"Success! Block hash: {response.block_hash.hex()}")


@guarded_command
def show_block_command(casperlabs_client, args):
    response = casperlabs_client.showBlock(args.hash, full_view=True)
    return _show_block(response)


@guarded_command
def show_blocks_command(casperlabs_client, args):
    response = casperlabs_client.showBlocks(args.depth)
    _show_blocks(response)


@guarded_command
def vdag_command(casperlabs_client, args):
    response = casperlabs_client.visualizeDag(args.depth)
    # TODO: call Graphviz
    print(hexify(response))


@guarded_command
def query_state_command(casperlabs_client, args):
    response = casperlabs_client.queryState(
        args.block_hash, args.key, args.path, getattr(args, "type")
    )
    print(hexify(response))


@guarded_command
def balance_command(casperlabs_client, args):
    response = casperlabs_client.balance(args.address, args.block_hash)
    print(response)


@guarded_command
def show_deploy_command(casperlabs_client, args):
    response = casperlabs_client.showDeploy(args.hash, full_view=False)
    print(hexify(response))


@guarded_command
def show_deploys_command(casperlabs_client, args):
    response = casperlabs_client.showDeploys(args.hash, full_view=False)
    _show_blocks(response, element_name="deploy")


# fmt: off
def deploy_options(keys_required=False, private_key_accepted=True):
    return ([
        [('-f', '--from'), dict(required=True, type=str, help="The public key of the account which is the context of this deployment, base16 encoded.")],
        [('--dependencies',), dict(required=False, nargs="+", default=None, help="List of deploy hashes (base16 encoded) which must be executed before this deploy.")],
        [('--payment-amount',), dict(required=False, type=int, default=None, help="Standard payment amount. Use this with the default payment, or override with --payment-args if custom payment code is used.")],
        [('--gas-price',), dict(required=False, type=int, default=10, help='The price of gas for this transaction in units dust/gas. Must be positive integer.')],
        [('-p', '--payment'), dict(required=False, type=str, default=None, help='Path to the file with payment code, by default fallbacks to the --session code')],
        [('--payment-hash',), dict(required=False, type=str, default=None, help='Hash of the stored contract to be called in the payment; base16 encoded')],
        [('--payment-name',), dict(required=False, type=str, default=None, help='Name of the stored contract (associated with the executing account) to be called in the payment')],
        [('--payment-uref',), dict(required=False, type=str, default=None, help='URef of the stored contract to be called in the payment; base16 encoded')],
        [('-s', '--session'), dict(required=False, type=str, default=None, help='Path to the file with session code')],
        [('--session-hash',), dict(required=False, type=str, default=None, help='Hash of the stored contract to be called in the session; base16 encoded')],
        [('--session-name',), dict(required=False, type=str, default=None, help='Name of the stored contract (associated with the executing account) to be called in the session')],
        [('--session-uref',), dict(required=False, type=str, default=None, help='URef of the stored contract to be called in the session; base16 encoded')],
        [('--session-args',), dict(required=False, type=str, help="""JSON encoded list of session args, e.g.: '[{"name": "amount", "value": {"long_value": 123456}}]'""")],
        [('--payment-args',), dict(required=False, type=str, help="""JSON encoded list of payment args, e.g.: '[{"name": "amount", "value": {"big_int": {"value": "123456", "bit_width": 512}}}]'""")],
        [('--ttl',), dict(required=False, type=int, help="""Time to live. Time (in milliseconds) that the deploy will remain valid for.'""")],
        [('--public-key',), dict(required=keys_required, default=None, type=str, help='Path to the file with account public key (Ed25519)')]]
        + (private_key_accepted
           and [[('--private-key',), dict(required=keys_required, default=None, type=str, help='Path to the file with account private key (Ed25519)')]]
           or []))
# fmt:on


def main():
    """
    Parse command line and call an appropriate command.
    """

    class Parser:
        def __init__(self):
            self.parser = argparse.ArgumentParser(add_help=False)
            self.parser.add_argument(
                "--help",
                action="help",
                default=argparse.SUPPRESS,
                help="show this help message and exit",
            )
            self.parser.add_argument(
                "-h",
                "--host",
                required=False,
                default=DEFAULT_HOST,
                type=str,
                help="Hostname or IP of node on which gRPC service is running.",
            )
            self.parser.add_argument(
                "-p",
                "--port",
                required=False,
                default=DEFAULT_PORT,
                type=int,
                help="Port used for external gRPC API.",
            )
            self.parser.add_argument(
                "--port-internal",
                required=False,
                default=DEFAULT_INTERNAL_PORT,
                type=int,
                help="Port used for internal gRPC API.",
            )
            self.parser.add_argument(
                "--node-id",
                required=False,
                type=str,
                help="node_id parameter for TLS connection",
            )
            self.parser.add_argument(
                "--certificate-file",
                required=False,
                type=str,
                help="Certificate file for TLS connection",
            )
            self.sp = self.parser.add_subparsers(help="Choose a request")

            self.parser.set_defaults(function=no_command)

        def addCommand(self, command: str, function, help, arguments):
            command_parser = self.sp.add_parser(command, help=help)
            command_parser.set_defaults(function=function)
            for (args, options) in arguments:
                command_parser.add_argument(*args, **options)

        def run(self):
            if len(sys.argv) < 2:
                self.parser.print_usage()
                return 1

            args = self.parser.parse_args()
            return args.function(
                CasperLabsClient(
                    args.host,
                    args.port,
                    args.port_internal,
                    args.node_id,
                    args.certificate_file,
                ),
                args,
            )

    parser = Parser()

    # fmt: off
    parser.addCommand('deploy', deploy_command, 'Deploy a smart contract source file to Casper on an existing running node. The deploy will be packaged and sent as a block to the network depending on the configuration of the Casper instance',
                      deploy_options(keys_required=True))

    parser.addCommand('make-deploy', make_deploy_command, "Constructs a deploy that can be signed and sent to a node.",
                      [[('-o', '--deploy-path'), dict(required=False, help="Path to the file where deploy will be saved. Optional, if not provided the deploy will be printed to STDOUT.")]] + deploy_options(keys_required=False, private_key_accepted=False))

    parser.addCommand('sign-deploy', sign_deploy_command, "Cryptographically signs a deploy. The signature is appended to existing approvals.",
                      [[('-o', '--signed-deploy-path'), dict(required=False, default=None, help="Path to the file where signed deploy will be saved. Optional, if not provided the deploy will be printed to STDOUT.")],
                       [('-i', '--deploy-path'), dict(required=False, default=None, help="Path to the deploy file.")],
                       [('--private-key',), dict(required=True, help="Path to the file with account private key (Ed25519)")],
                       [('--public-key',), dict(required=True, help="Path to the file with account public key (Ed25519)")]])

    parser.addCommand('send-deploy', send_deploy_command, "Deploy a smart contract source file to Casper on an existing running node. The deploy will be packaged and sent as a block to the network depending on the configuration of the Casper instance.",
                      [[('-i', '--deploy-path'), dict(required=False, default=None, help="Path to the file with signed deploy.")]])

    parser.addCommand('bond', bond_command, 'Issues bonding request',
                      [[('-a', '--amount'), dict(required=True, type=int, help='amount of motes to bond')]] + deploy_options(keys_required=True))

    parser.addCommand('unbond', unbond_command, 'Issues unbonding request',
                      [[('-a', '--amount'),
                       dict(required=False, default=None, type=int, help='Amount of motes to unbond. If not provided then a request to unbond with full staked amount is made.')]] + deploy_options(keys_required=True))

    parser.addCommand('propose', propose_command, 'Force a node to propose a block based on its accumulated deploys.', [])

    parser.addCommand('show-block', show_block_command, 'View properties of a block known by Casper on an existing running node. Output includes: parent hashes, storage contents of the tuplespace.',
                      [[('hash',), dict(type=str, help='the hash value of the block')]])

    parser.addCommand('show-blocks', show_blocks_command, 'View list of blocks in the current Casper view on an existing running node.',
                      [[('-d', '--depth'), dict(required=True, type=int, help='depth in terms of block height')]])

    parser.addCommand('show-deploy', show_deploy_command, 'View properties of a deploy known by Casper on an existing running node.',
                      [[('hash',), dict(type=str, help='Value of the deploy hash, base16 encoded.')]])

    parser.addCommand('show-deploys', show_deploys_command, 'View deploys included in a block.',
                      [[('hash',), dict(type=str, help='Value of the block hash, base16 encoded.')]])

    parser.addCommand('vdag', vdag_command, 'DAG in DOT format',
                      [[('-d', '--depth'), dict(required=True, type=int, help='depth in terms of block height')],
                       [('-o', '--out'), dict(required=False, type=str, help='output image filename, outputs to stdout if not specified, must end with one of the png, svg, svg_standalone, xdot, plain, plain_ext, ps, ps2, json, json0')],
                       [('-s', '--show-justification-lines'), dict(action='store_true', help='if justification lines should be shown')],
                       [('--stream',), dict(required=False, choices=('single-output', 'multiple-outputs'), help="subscribe to changes, '--out' has to be specified, valid values are 'single-output', 'multiple-outputs'")]])

    parser.addCommand('query-state', query_state_command, 'Query a value in the global state.',
                      [[('-b', '--block-hash'), dict(required=True, type=str, help='Hash of the block to query the state of')],
                       [('-k', '--key'), dict(required=True, type=str, help='Base16 encoding of the base key')],
                       [('-p', '--path'), dict(required=True, type=str, help="Path to the value to query. Must be of the form 'key1/key2/.../keyn'")],
                       [('-t', '--type'), dict(required=True, choices=('hash', 'uref', 'address', 'local'),
                                               help="Type of base key. Must be one of 'hash', 'uref', 'address' or 'local'. For 'local' key type, 'key' value format is {seed}:{rest}, where both parts are hex encoded.")]])

    parser.addCommand('balance', balance_command, 'Returns the balance of the account at the specified block.',
                      [[('-a', '--address'), dict(required=True, type=str, help="Account's public key in hex.")],
                       [('-b', '--block-hash'), dict(required=True, type=str, help='Hash of the block to query the state of')]])
    # fmt:on
    sys.exit(parser.run())


def check_bundled_contracts():
    print(dir(pkg_resources))
    p = pkg_resources.resource_filename(__name__, "bonding.wasm")
    if not os.path.exists(p):
        raise Exception(f"No bundled contract {p}")


if __name__ == "__main__":
    main()
